Imports System.IO
Imports System.Web.Services
Imports System.Web.Services.Protocols
Imports System.Xml
Imports System.Xml.Xsl

Public Class SoapDisplayExtension
    Inherits SoapExtension

    ' Variables to hold the original stream
    ' and the stream that we return
    Private originalStream As Stream
    Private internalStream As Stream

    ' Called the first time the Web service is used
    ' Version called if configured with an attribute
    Public Overloads Overrides Function GetInitializer( _
     ByVal methodInfo As LogicalMethodInfo, _
     ByVal attribute As SoapExtensionAttribute) As Object
        ' Not used in this example, but it's declared 
        ' MustOverride in the base class
    End Function
    ' Version called if configured with a config file
    Public Overloads Overrides Function GetInitializer( _
    ByVal WebServiceType As Type) As Object
        ' Not used in this example, but it's declared 
        ' MustOverride in the base class
    End Function

    ' Called each time the Web service is used
    ' And gets passed the data from GetInitializer
    Public Overrides Sub Initialize( _
     ByVal initializer As Object)
        ' Not used in this example, but it's declared 
        ' MustOverride in the base class
    End Sub

    ' The Chainstream method gives us a chance
    ' to grab the SOAP messages as they go by
    Public Overrides Function ChainStream( _
     ByVal stream As Stream) As Stream
        ' Save the original stream
        originalStream = stream
        ' Create and return our own in its place
        internalStream = New MemoryStream()
        ChainStream = internalStream
    End Function

    ' The ProcessMessage method is where we do our work
    Public Overrides Sub ProcessMessage( _
     ByVal message As SoapMessage)
        ' Determine the stage and take appropriate action
        Select Case message.Stage
            Case SoapMessageStage.BeforeSerialize
                ' About to prepare a SOAP request
            Case SoapMessageStage.AfterSerialize
                ' SOAP request is all prepared
                internalStream.Position = 0
                ' Get a Transform ready
                Dim xslt As XslTransform = New XslTransform()
                xslt.Load("..\identity.xslt")
                ' Load the raw XML into an XML document
                Dim xd As XmlDocument = New XmlDocument()
                xd.Load(internalStream)
                ' Use the transform to pretty print it 
                Dim ms As MemoryStream = New MemoryStream()
                xslt.Transform(xd, Nothing, ms)
                ' And drop the results to a TextBox control
                ms.Position = 0
                Dim sr As StreamReader = New StreamReader(ms)
                Dim f As New DisplayMessage()
                f.txtMessage.Text = sr.ReadToEnd()
                f.Show()
                ' Copy the passed message to the other stream
                internalStream.Position = 0
                CopyStream(internalStream, originalStream)
                internalStream.Position = 0
            Case SoapMessageStage.BeforeDeserialize
                ' About to handle a SOAP response
                ' Copy the passed message to the other stream
                CopyStream(originalStream, internalStream)
                internalStream.Position = 0
                ' Get a Transform ready
                Dim xslt As XslTransform = New XslTransform()
                xslt.Load("..\identity.xslt")
                ' Load the raw XML into an XML document
                Dim xd As XmlDocument = New XmlDocument()
                xd.Load(internalStream)
                ' Use the transform to pretty print it 
                Dim ms As MemoryStream = New MemoryStream()
                xslt.Transform(xd, Nothing, ms)
                ' And drop the results to a TextBox control
                ms.Position = 0
                Dim sr As StreamReader = New StreamReader(ms)
                Dim f As New DisplayMessage()
                f.txtMessage.Text = sr.ReadToEnd()
                f.Show()
                internalStream.Position = 0
            Case SoapMessageStage.AfterDeserialize
                ' SOAP response has been deserialized
        End Select
    End Sub

    ' Helper function to copy one stream to another
    Private Sub CopyStream(ByVal fromStream As Stream, _
    ByVal toStream As Stream)
        Try
            Dim sr As New StreamReader(fromStream)
            Dim sw As New StreamWriter(toStream)
            sw.WriteLine(sr.ReadToEnd())
            sw.Flush()
        Catch ex As Exception
        End Try
    End Sub
End Class
