Imports System
Imports System.Data
Imports System.Data.SqlClient
Imports System.EnterpriseServices
Imports System.Runtime.InteropServices
Imports System.Diagnostics

Public Interface INorthwind
    Function ExecuteQuery(ByVal strQuery As String) _
     As DataSet
    Function UpdateData(ByVal ds As DataSet) As Integer
End Interface

<ObjectPooling(True, 2, 4), _
 ClassInterface(ClassInterfaceType.None), _
 Guid("CE1F97FD-607E-4cee-8BC9-4B3883DE5D3B")> _
 Public Class NorthwindSC
    Inherits ServicedComponent
    Implements INorthwind

    Private sqlcnn As SqlConnection
    Private sqlda As SqlDataAdapter
    Private ds As DataSet
    Private el As EventLog

    Public Sub New()
        ' Perform expensive startup operations

        ' Create a connection to the 
        ' Northwind SQL Server database
        sqlcnn = New SqlConnection( _
         "data source=(local);" & _
         "initial catalog=Northwind;" & _
         "integrated security=SSPI")

        ' Create an EventLog object 
        ' and assign its source
        el = New EventLog()
        el.Source = "NorthwindPool"

        ' Write an entry to the event log    
        el.WriteEntry( _
         "NorthwindSC object is created" & _
        " and added to the object pool")

    End Sub

    Protected Overrides Function CanBePooled() As Boolean
        CanBePooled = True
    End Function

    Protected Overrides Sub Activate()
        el.WriteEntry( _
         "A NorthwindSC object is activated" & _
         " from the object pool")
    End Sub

    Protected Overrides Sub Deactivate()
        el.WriteEntry( _
         "A NorthwindSC object is deactivated" & _
         " and is returned to the object pool")
    End Sub

    ' This method executes a SELECT query and
    ' returns the results in a DataSet object
    Public Function ExecuteQuery( _
     ByVal strQuery As String) As DataSet _
     Implements INorthwind.ExecuteQuery
        ' Create a SqlDataAdapter object to 
        ' talk to the database
        sqlda = New SqlDataAdapter( _
         strQuery, sqlcnn)

        ' Create a DataSet object 
        ' to hold the results
        ds = New DataSet()

        ' Fill the DataSet object
        sqlda.Fill(ds, "Results")

        ExecuteQuery = ds
    End Function

    ' This method updates the database with
    ' the changes in a DataSet object
    Public Function UpdateData( _
     ByVal ds As DataSet) As Integer _
     Implements INorthwind.UpdateData
        ' Update the database 
        ' and return the result
        Dim sqlcb As SqlCommandBuilder = _
         New SqlCommandBuilder(sqlda)
        UpdateData = sqlda.Update( _
         ds.Tables("Results"))
    End Function
End Class
